-- diskdemo.ex
-- Test run of some routines in the include file 'diskutil.e' (some
-- indirectly).
-- J. Craig Gilbert
-- cgilbert@cennet.mc.peachnet.edu
include diskutil.e

------------------------------------------------------------------------------
function trim(sequence s, atom trimchar)
-- trim all instances of given character from both ends of sequence s
integer len, pos

   len = length(s)
   if len then
      pos = 1
      while s[pos] = trimchar do
         if pos = len then
            exit
         end if
         pos = pos + 1
      end while

      s = s[pos..len]
      len = length(s)
      pos = len
      while s[pos] = trimchar do
         if pos = 1 then
            exit
         end if
         pos = pos - 1
      end while
      s = s[1..pos]
      if length(s) = 1 then
         if s[1] = trimchar then
            s = ""
         end if
      end if
   end if

   return s
end function
------------------------------------------------------------------------------
constant LETTER = 1, READY = 2, WRITE = 3, FREE = 4, LABEL = 5, SERIAL = 6,
         FILESYS = 7
procedure main()
object temp
sequence drives, drive_data
integer status

   clear_screen()

   drives = drive_list()
   puts(1,"DRIVE  READY  WRITE-PROTECT  FREE SPACE     LABEL     SERIAL NO  FILE SYSTEM\n")
   puts(1,"-----  -----  -------------  ----------  -----------  ---------  -----------\n")
   for letter = 1 to length(drives) do
      drive_data = repeat(0,7)
      drive_data[LETTER] = {upper(drives[letter])}
      status = check_drive(drives[letter])
      if status = DRIVE_READY or status = DRIVE_WRITE_PROTECTED then
         drive_data[READY] = "YES"
         if status = DRIVE_READY then
            drive_data[WRITE] = "NO"
         else
            drive_data[WRITE] = "YES"
         end if
         drive_data[FREE] = sprintf("%10.0f",{disk_free_space(drives[letter])})
         temp = get_disk_id(drives[letter])
         if sequence(temp) then
            drive_data[LABEL] = trim(temp[2], ' ')
            drive_data[SERIAL] = temp[1]
            drive_data[FILESYS] = trim(temp[3], ' ')
         else
            drive_data[SERIAL] = sprintf("ERR:%d",{temp}) -- DOS error code
            drive_data[LABEL] = sprintf("ERR:%d",{temp}) -- DOS error code
            drive_data[FILESYS] = sprintf("ERR:%d",{temp}) -- DOS error code
         end if
      elsif status = DRIVE_NOT_READY then
         drive_data[READY] = "NO"
         drive_data[WRITE..FILESYS] = repeat("-",5)
      elsif status = DRIVE_NOT_EXIST then
         drive_data[READY..FILESYS] = repeat("-",6)
      else -- unusual dos error code returned (general failure, etc)
         drive_data[READY] = sprintf("%d",{status})
         drive_data[WRITE..FILESYS] = repeat("!",5)
      end if

      printf(1,"%+3s%+8s%+11s%+17s%+13s%+11s%+13s\n",{drive_data[LETTER],
            drive_data[READY], drive_data[WRITE], drive_data[FREE],
            drive_data[LABEL], drive_data[SERIAL], drive_data[FILESYS]})
   end for

   puts(1,"\n\nIn addition to gathering the information above,\n")
   puts(1,"diskutil.e also contains routines for:\n")
   puts(1,"   - detecting/setting the current default drive\n")
   puts(1,"   - verifying existence of drive\n")
   puts(1,"   - setting the volume label of a disk\n")
   puts(1,"   - setting the serial number of a disk\n\n")
end procedure

main()

