-- mach.e
-- some conveniences for working with machine code and interrupt handlers
-- J. Craig Gilbert
-- cgilbert@cennet.mc.peachnet.edu

include machine.e
------------------------------------------------------------------------------
global function mem_locked(integer amount)
-- allocate amount of memory and lock it
atom addr
   addr = allocate(amount)
   if addr != 0 then -- if allocation successful
      lock_memory(addr,amount)
   end if

   return addr
end function
------------------------------------------------------------------------------
global function get_seg(integer seg_type)
-- get current segment:
-- code seg if set_type is 0
-- otherwise data seg
-- returns an int, use int_to_bytes() before using in machine code
atom mem_ptr, code_ptr, seg
integer push_byte
sequence get_seg_code
   if seg_type = 0 then
      push_byte = #0E   -- push cs
   else
      push_byte = #1E   -- push ds
   end if
   mem_ptr = mem_locked(4)

   get_seg_code = { #53}   -- push ebx
                  & push_byte & -- push cs/ds
                  { #5B,   -- pop ebx
                    #89, #1D} & int_to_bytes(mem_ptr) & -- mov mem, ebx
                  { #5B,   -- pop ebx
                    #C3    -- ret
                  }

   code_ptr = allocate(length(get_seg_code))
   poke(code_ptr, get_seg_code)
   call(code_ptr)
   seg = peek(mem_ptr) + 256 * peek(mem_ptr+1)

   free(code_ptr)
   free(mem_ptr)

return seg
end function
------------------------------------------------------------------------------
global function far_abs_addr(sequence segoff)
-- takes a 2 element sequence {16 bit segment, 32 bit address}
-- returns a 6 element sequence representing the equivalent far address
-- in machine code form
return   int_to_bytes(segoff[2]) & 
         remainder(segoff[1],256) & floor(segoff[1]/256)
end function
------------------------------------------------------------------------------
sequence mhandlers
mhandlers = {}
global function install_handler(sequence new, integer interrupt)
-- Returns 0 if problem, address of new handler if installed
atom new_addr
atom code_segment
sequence old_int
   
   new_addr = mem_locked(length(new))
   if new_addr = 0 then
      return 0
   end if
   poke(new_addr, new)
   
   old_int = get_vector(interrupt)
   mhandlers = {interrupt, {new_addr, old_int} } & mhandlers
   
   code_segment = get_seg(0)
   set_vector(interrupt, {code_segment, new_addr})

   return new_addr
end function
------------------------------------------------------------------------------
global procedure reset_handler(integer interrupt)
-- given the interrupt number, restore the previous interrupt handler
-- and free the memory taken by the previous handler
integer found, len
   found = find(interrupt, mhandlers)
   if found then
      set_vector(interrupt, mhandlers[found+1][2])
      free(mhandlers[found+1][1])
      len = length(mhandlers)
      if found = 1 then
         if len > 2 then
            mhandlers = mhandlers[3..len]
         else
            mhandlers = {}
         end if
      elsif found = len - 1 then
         if len > 2 then
            mhandlers = mhandlers[1..len-2]
         else
            mhandlers = {}
         end if
      else
         mhandlers = mhandlers[1..found-1] & mhandlers[found+2..len]
      end if
   end if
return
end procedure
------------------------------------------------------------------------------

